/* ###*B*###
 * ERIKA Enterprise - a tiny RTOS for small microcontrollers
 *
 * Copyright (C) 2002-2016  Evidence Srl
 *
 * This file is part of ERIKA Enterprise.
 *
 * ERIKA Enterprise is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation,
 * (with a special exception described below).
 *
 * Linking this code statically or dynamically with other modules is
 * making a combined work based on this code.  Thus, the terms and
 * conditions of the GNU General Public License cover the whole
 * combination.
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this code with independent modules to produce an
 * executable, regardless of the license terms of these independent
 * modules, and to copy and distribute the resulting executable under
 * terms of your choice, provided that you also meet, for each linked
 * independent module, the terms and conditions of the license of that
 * module.  An independent module is a module which is not derived from
 * or based on this library.  If you modify this code, you may extend
 * this exception to your version of the code, but you are not
 * obligated to do so.  If you do not wish to do so, delete this
 * exception statement from your version.
 *
 * ERIKA Enterprise is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License version 2 for more details.
 *
 * You should have received a copy of the GNU General Public License
 * version 2 along with ERIKA Enterprise; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 * ###*E*### */

/*
 * Atmel ATMEGA MCUs Interrupts vector.
 *
 * Author: 2016,  Giuseppe Serano
 */

/* I need ISR2 Stub */
#include "cpu/avr8/inc/ee_avr8_irqstub.h"

/* Erika Interrupt Vector Definition */

/* DUMMY interrupt vector */
EE_AVR8_ISR_NOT_DEFINED(BADISR_vect)

/* Interrupt Vector Table Recall. */
void EE_atmega_intvect(void) {
  EE_AVR8_REQUEST_IRQ_ENTRY(BADISR_vect);
}

/* INT0 interrupt vector */
#if defined(EE_AVR8_INT0_ISR) && (EE_AVR8_INT0_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(INT0_vect, EE_AVR8_INT0_ISR)
#elif defined(EE_AVR8_INT0_ISR) && (EE_AVR8_INT0_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(INT0_vect, EE_AVR8_INT0_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(INT0_vect)
#endif /* EE_AVR8_INT0_ISR */

/* INT1 interrupt vector */
#if defined(EE_AVR8_INT1_ISR) && (EE_AVR8_INT1_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(INT1_vect, EE_AVR8_INT1_ISR)
#elif defined(EE_AVR8_INT1_ISR) && (EE_AVR8_INT1_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(INT1_vect, EE_AVR8_INT1_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(INT1_vect)
#endif /* EE_AVR8_INT1_ISR */

#if	( defined(__AVR_ATmega128__) || defined(__AVR_ATmega1281__) )
/* INT2 interrupt vector */
#if defined(EE_AVR8_INT2_ISR) && (EE_AVR8_INT2_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(INT2_vect, EE_AVR8_INT2_ISR)
#elif defined(EE_AVR8_INT2_ISR) && (EE_AVR8_INT2_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(INT2_vect, EE_AVR8_INT2_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(INT2_vect)
#endif /* EE_AVR8_INT2_ISR */

/* INT3 interrupt vector */
#if defined(EE_AVR8_INT3_ISR) && (EE_AVR8_INT3_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(INT3_vect, EE_AVR8_INT3_ISR)
#elif defined(EE_AVR8_INT3_ISR) && (EE_AVR8_INT3_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(INT3_vect, EE_AVR8_INT3_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(INT3_vect)
#endif /* EE_AVR8_INT3_ISR */

/* INT4 interrupt vector */
#if defined(EE_AVR8_INT4_ISR) && (EE_AVR8_INT4_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(INT4_vect, EE_AVR8_INT4_ISR)
#elif defined(EE_AVR8_INT4_ISR) && (EE_AVR8_INT4_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(INT4_vect, EE_AVR8_INT4_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(INT4_vect)
#endif /* EE_AVR8_INT4_ISR */

/* INT5 interrupt vector */
#if defined(EE_AVR8_INT5_ISR) && (EE_AVR8_INT5_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(INT5_vect, EE_AVR8_INT5_ISR)
#elif defined(EE_AVR8_INT5_ISR) && (EE_AVR8_INT5_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(INT5_vect, EE_AVR8_INT5_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(INT5_vect)
#endif /* EE_AVR8_INT5_ISR */

/* INT6 interrupt vector */
#if defined(EE_AVR8_INT6_ISR) && (EE_AVR8_INT6_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(INT6_vect, EE_AVR8_INT6_ISR)
#elif defined(EE_AVR8_INT6_ISR) && (EE_AVR8_INT6_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(INT6_vect, EE_AVR8_INT6_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(INT6_vect)
#endif /* EE_AVR8_INT6_ISR */

/* INT7 interrupt vector */
#if defined(EE_AVR8_INT7_ISR) && (EE_AVR8_INT7_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(INT7_vect, EE_AVR8_INT7_ISR)
#elif defined(EE_AVR8_INT7_ISR) && (EE_AVR8_INT7_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(INT7_vect, EE_AVR8_INT7_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(INT7_vect)
#endif /* EE_AVR8_INT7_ISR */
#endif	/* __AVR_ATmega128__ || __AVR_ATmega1281__ */

#if	(			   \
  defined(__AVR_ATmega328__)	|| \
  defined(__AVR_ATmega328P__)	|| \
  defined(__AVR_ATmega1281__)	   \
)
/* PCINT0 interrupt vector */
#if defined(EE_AVR8_PCINT0_ISR) && (EE_AVR8_PCINT0_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(PCINT0_vect, EE_AVR8_PCINT0_ISR)
#elif defined(EE_AVR8_PCINT0_ISR) && (EE_AVR8_PCINT0_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(PCINT0_vect, EE_AVR8_PCINT0_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(PCINT0_vect)
#endif /* EE_AVR8_PCINT0_ISR */

/* PCINT1 interrupt vector */
#if defined(EE_AVR8_PCINT1_ISR) && (EE_AVR8_PCINT1_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(PCINT1_vect, EE_AVR8_PCINT1_ISR)
#elif defined(EE_AVR8_PCINT1_ISR) && (EE_AVR8_PCINT1_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(PCINT1_vect, EE_AVR8_PCINT1_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(PCINT1_vect)
#endif /* EE_AVR8_PCINT1_ISR */

/* PCINT2 interrupt vector */
#if defined(EE_AVR8_PCINT2_ISR) && (EE_AVR8_PCINT2_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(PCINT2_vect, EE_AVR8_PCINT2_ISR)
#elif defined(EE_AVR8_PCINT2_ISR) && (EE_AVR8_PCINT2_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(PCINT2_vect, EE_AVR8_PCINT2_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(PCINT2_vect)
#endif /* EE_AVR8_PCINT2_ISR */

/* WDT interrupt vector */
#if defined(EE_AVR8_WDT_ISR) && (EE_AVR8_WDT_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(WDT_vect, EE_AVR8_WDT_ISR)
#elif defined(EE_AVR8_WDT_ISR) && (EE_AVR8_WDT_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(WDT_vect, EE_AVR8_WDT_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(WDT_vect)
#endif /* EE_AVR8_WDT_ISR */
#endif	/* __AVR_ATmega328__ || __AVR_ATmega328P__ || __AVR_ATmega1281__ */

#ifdef	__AVR_ATmega128__
/* TIMER2_COMP interrupt vector */
#if defined(EE_AVR8_TIMER2_COMP_ISR) && (EE_AVR8_TIMER2_COMP_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER2_COMP_vect, EE_AVR8_TIMER2_COMP_ISR)
#elif defined(EE_AVR8_TIMER2_COMP_ISR) && (EE_AVR8_TIMER2_COMP_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER2_COMP_vect, EE_AVR8_TIMER2_COMP_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER2_COMP_vect)
#endif /* EE_AVR8_TIMER2_COMP_ISR */
#endif	/* __AVR_ATmega128__ */

#if	(			   \
  defined(__AVR_ATmega328__)	|| \
  defined(__AVR_ATmega328P__)	|| \
  defined(__AVR_ATmega1281__)	   \
)
/* TIMER2_COMPA interrupt vector */
#if defined(EE_AVR8_TIMER2_COMPA_ISR) && (EE_AVR8_TIMER2_COMPA_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER2_COMPA_vect, EE_AVR8_TIMER2_COMPA_ISR)
#elif defined(EE_AVR8_TIMER2_COMPA_ISR) && (EE_AVR8_TIMER2_COMPA_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER2_COMPA_vect, EE_AVR8_TIMER2_COMPA_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER2_COMPA_vect)
#endif /* EE_AVR8_TIMER2_COMPA_ISR */

/* TIMER2_COMPB interrupt vector */
#if defined(EE_AVR8_TIMER2_COMPB_ISR) && (EE_AVR8_TIMER2_COMPB_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER2_COMPB_vect, EE_AVR8_TIMER2_COMPB_ISR)
#elif defined(EE_AVR8_TIMER2_COMPB_ISR) && (EE_AVR8_TIMER2_COMPB_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER2_COMPB_vect, EE_AVR8_TIMER2_COMPB_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER2_COMPB_vect)
#endif /* EE_AVR8_TIMER2_COMPB_ISR */
#endif	/* __AVR_ATmega328__ || __AVR_ATmega328P__ || __AVR_ATmega1281__ */

/* TIMER2_OVF interrupt vector */
#if defined(EE_AVR8_TIMER2_OVF_ISR) && (EE_AVR8_TIMER2_OVF_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER2_OVF_vect, EE_AVR8_TIMER2_OVF_ISR)
#elif defined(EE_AVR8_TIMER2_OVF_ISR) && (EE_AVR8_TIMER2_OVF_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER2_OVF_vect, EE_AVR8_TIMER2_OVF_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER2_OVF_vect)
#endif /* EE_AVR8_TIMER2_OVF_ISR */

/* TIMER1_CAPT interrupt vector */
#if defined(EE_AVR8_TIMER1_CAPT_ISR) && (EE_AVR8_TIMER1_CAPT_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER1_CAPT_vect, EE_AVR8_TIMER1_CAPT_ISR)
#elif defined(EE_AVR8_TIMER1_CAPT_ISR) && (EE_AVR8_TIMER1_CAPT_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER1_CAPT_vect, EE_AVR8_TIMER1_CAPT_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER1_CAPT_vect)
#endif /* EE_AVR8_TIMER1_CAPT_ISR */

/* TIMER1_COMPA interrupt vector */
#if defined(EE_AVR8_TIMER1_COMPA_ISR) && (EE_AVR8_TIMER1_COMPA_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER1_COMPA_vect, EE_AVR8_TIMER1_COMPA_ISR)
#elif defined(EE_AVR8_TIMER1_COMPA_ISR) && (EE_AVR8_TIMER1_COMPA_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER1_COMPA_vect, EE_AVR8_TIMER1_COMPA_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER1_COMPA_vect)
#endif /* EE_AVR8_TIMER1_COMPA_ISR */

/* TIMER1_COMPB interrupt vector */
#if defined(EE_AVR8_TIMER1_COMPB_ISR) && (EE_AVR8_TIMER1_COMPB_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER1_COMPB_vect, EE_AVR8_TIMER1_COMPB_ISR)
#elif defined(EE_AVR8_TIMER1_COMPB_ISR) && (EE_AVR8_TIMER1_COMPB_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER1_COMPB_vect, EE_AVR8_TIMER1_COMPB_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER1_COMPB_vect)
#endif /* EE_AVR8_TIMER1_COMPB_ISR */

#if	( defined(__AVR_ATmega128__) || defined(__AVR_ATmega1281__) )
/* TIMER1_COMPC interrupt vector */
#if defined(EE_AVR8_TIMER1_COMPC_ISR) && (EE_AVR8_TIMER1_COMPC_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER1_COMPC_vect, EE_AVR8_TIMER1_COMPC_ISR)
#elif defined(EE_AVR8_TIMER1_COMPC_ISR) && (EE_AVR8_TIMER1_COMPC_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER1_COMPC_vect, EE_AVR8_TIMER1_COMPC_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER1_COMPC_vect)
#endif /* EE_AVR8_TIMER1_COMPC_ISR */
#endif	/* __AVR_ATmega128__ || __AVR_ATmega1281__ */

/* TIMER1_OVF interrupt vector */
#if defined(EE_AVR8_TIMER1_OVF_ISR) && (EE_AVR8_TIMER1_OVF_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER1_OVF_vect, EE_AVR8_TIMER1_OVF_ISR)
#elif defined(EE_AVR8_TIMER1_OVF_ISR) && (EE_AVR8_TIMER1_OVF_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER1_OVF_vect, EE_AVR8_TIMER1_OVF_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER1_OVF_vect)
#endif /* EE_AVR8_TIMER1_OVF_ISR */

#ifdef	__AVR_ATmega128__
/* TIMER0_COMP interrupt vector */
#if defined(EE_AVR8_TIMER0_COMP_ISR) && (EE_AVR8_TIMER0_COMP_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER0_COMP_vect, EE_AVR8_TIMER0_COMP_ISR)
#elif defined(EE_AVR8_TIMER0_COMP_ISR) && (EE_AVR8_TIMER0_COMP_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER0_COMP_vect, EE_AVR8_TIMER0_COMP_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER0_COMP_vect)
#endif /* EE_AVR8_TIMER0_COMP_ISR */
#endif	/* __AVR_ATmega128__ */

#if	(			   \
  defined(__AVR_ATmega328__)	|| \
  defined(__AVR_ATmega328P__)	|| \
  defined(__AVR_ATmega1281__)	   \
)
/* TIMER2_COMPA interrupt vector */
#if defined(EE_AVR8_TIMER0_COMPA_ISR) && (EE_AVR8_TIMER0_COMPA_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER0_COMPA_vect, EE_AVR8_TIMER0_COMPA_ISR)
#elif defined(EE_AVR8_TIMER0_COMPA_ISR) && (EE_AVR8_TIMER0_COMPA_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER0_COMPA_vect, EE_AVR8_TIMER0_COMPA_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER0_COMPA_vect)
#endif /* EE_AVR8_TIMER0_COMPA_ISR */

/* TIMER0_COMPB interrupt vector */
#if defined(EE_AVR8_TIMER0_COMPB_ISR) && (EE_AVR8_TIMER0_COMPB_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER0_COMPB_vect, EE_AVR8_TIMER0_COMPB_ISR)
#elif defined(EE_AVR8_TIMER0_COMPB_ISR) && (EE_AVR8_TIMER0_COMPB_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER0_COMPB_vect, EE_AVR8_TIMER0_COMPB_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER0_COMPB_vect)
#endif /* EE_AVR8_TIMER0_COMPB_ISR */
#endif	/* __AVR_ATmega328__ || __AVR_ATmega328P__ || __AVR_ATmega1281__ */

/* TIMER0_OVF interrupt vector */
#if defined(EE_AVR8_TIMER0_OVF_ISR) && (EE_AVR8_TIMER0_OVF_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER0_OVF_vect, EE_AVR8_TIMER0_OVF_ISR)
#elif defined(EE_AVR8_TIMER0_OVF_ISR) && (EE_AVR8_TIMER0_OVF_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER0_OVF_vect, EE_AVR8_TIMER0_OVF_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER0_OVF_vect)
#endif /* EE_AVR8_TIMER0_OVF_ISR */

/* SPI_STC interrupt vector */
#if defined(EE_AVR8_SPI_STC_ISR) && (EE_AVR8_SPI_STC_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(SPI_STC_vect, EE_AVR8_SPI_STC_ISR)
#elif defined(EE_AVR8_SPI_STC_ISR) && (EE_AVR8_SPI_STC_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(SPI_STC_vect, EE_AVR8_SPI_STC_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(SPI_STC_vect)
#endif /* EE_AVR8_SPI_STC_ISR */

#if	( defined(__AVR_ATmega128__) || defined(__AVR_ATmega1281__) )
/* USART0_RX interrupt vector */
#if defined(EE_AVR8_USART0_RX_ISR) && (EE_AVR8_USART0_RX_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(USART0_RX_vect, EE_AVR8_USART0_RX_ISR)
#elif defined(EE_AVR8_USART0_RX_ISR) && (EE_AVR8_USART0_RX_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(USART0_RX_vect, EE_AVR8_USART0_RX_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(USART0_RX_vect)
#endif /* EE_AVR8_USART0_RX_ISR */

/* USART0_UDRE interrupt vector */
#if defined(EE_AVR8_USART0_UDRE_ISR) && (EE_AVR8_USART0_UDRE_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(USART0_UDRE_vect, EE_AVR8_USART0_UDRE_ISR)
#elif defined(EE_AVR8_USART0_UDRE_ISR) && (EE_AVR8_USART0_UDRE_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(USART0_UDRE_vect, EE_AVR8_USART0_UDRE_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(USART0_UDRE_vect)
#endif /* EE_AVR8_USART0_UDRE_ISR */

/* USART0_TX interrupt vector */
#if defined(EE_AVR8_USART0_TX_ISR) && (EE_AVR8_USART0_TX_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(USART0_TX_vect, EE_AVR8_USART0_TX_ISR)
#elif defined(EE_AVR8_USART0_TX_ISR) && (EE_AVR8_USART0_TX_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(USART0_TX_vect, EE_AVR8_USART0_TX_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(USART0_TX_vect)
#endif /* EE_AVR8_USART0_TX_ISR */
#endif	/* __AVR_ATmega128__ || __AVR_ATmega1281__ */

#if	( defined (__AVR_ATmega328__) || defined(__AVR_ATmega328P__) )
/* USART_RX interrupt vector */
#if defined(EE_AVR8_USART_RX_ISR) && (EE_AVR8_USART_RX_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(USART_RX_vect, EE_AVR8_USART_RX_ISR)
#elif defined(EE_AVR8_USART_RX_ISR) && (EE_AVR8_USART_RX_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(USART_RX_vect, EE_AVR8_USART_RX_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(USART_RX_vect)
#endif /* EE_AVR8_USART_RX_ISR */

/* USART_UDRE interrupt vector */
#if defined(EE_AVR8_USART_UDRE_ISR) && (EE_AVR8_USART_UDRE_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(USART_UDRE_vect, EE_AVR8_USART_UDRE_ISR)
#elif defined(EE_AVR8_USART_UDRE_ISR) && (EE_AVR8_USART_UDRE_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(USART_UDRE_vect, EE_AVR8_USART_UDRE_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(USART_UDRE_vect)
#endif /* EE_AVR8_USART_UDRE_ISR */

/* USART_TX interrupt vector */
#if defined(EE_AVR8_USART_TX_ISR) && (EE_AVR8_USART_TX_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(USART_TX_vect, EE_AVR8_USART_TX_ISR)
#elif defined(EE_AVR8_USART_TX_ISR) && (EE_AVR8_USART_TX_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(USART_TX_vect, EE_AVR8_USART_TX_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(USART_TX_vect)
#endif /* EE_AVR8_USART_TX_ISR */
#endif	/* __AVR_ATmega328__ || __AVR_ATmega328P__ */

/* ADC interrupt vector */
#if defined(EE_AVR8_ADC_ISR) && (EE_AVR8_ADC_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(ADC_vect, EE_AVR8_ADC_ISR)
#elif defined(EE_AVR8_ADC_ISR) && (EE_AVR8_ADC_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(ADC_vect, EE_AVR8_ADC_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(ADC_vect)
#endif /* EE_AVR8_ADC_ISR */

/* EE_READY interrupt vector */
#if defined(EE_AVR8_EE_READY_ISR) && (EE_AVR8_EE_READY_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(EE_READY_vect, EE_AVR8_EE_READY_ISR)
#elif defined(EE_AVR8_EE_READY_ISR) && (EE_AVR8_EE_READY_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(EE_READY_vect, EE_AVR8_EE_READY_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(EE_READY_vect)
#endif /* EE_AVR8_EE_READY_ISR */

/* ANALOG_COMP interrupt vector */
#if defined(EE_AVR8_ANALOG_COMP_ISR) && (EE_AVR8_ANALOG_COMP_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(ANALOG_COMP_vect, EE_AVR8_ANALOG_COMP_ISR)
#elif defined(EE_AVR8_ANALOG_COMP_ISR) && (EE_AVR8_ANALOG_COMP_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(ANALOG_COMP_vect, EE_AVR8_ANALOG_COMP_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(ANALOG_COMP_vect)
#endif /* EE_AVR8_ANALOG_COMP_ISR */

#if	( defined(__AVR_ATmega128__) || defined(__AVR_ATmega1281__) )
/* TIMER3_CAPT interrupt vector */
#if defined(EE_AVR8_TIMER3_CAPT_ISR) && (EE_AVR8_TIMER3_CAPT_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER3_CAPT_vect, EE_AVR8_TIMER3_CAPT_ISR)
#elif defined(EE_AVR8_TIMER3_CAPT_ISR) && (EE_AVR8_TIMER3_CAPT_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER3_CAPT_vect, EE_AVR8_TIMER3_CAPT_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER3_CAPT_vect)
#endif /* EE_AVR8_TIMER3_CAPT_ISR */

/* TIMER3_COMPA interrupt vector */
#if defined(EE_AVR8_TIMER3_COMPA_ISR) && (EE_AVR8_TIMER3_COMPA_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER3_COMPA_vect, EE_AVR8_TIMER3_COMPA_ISR)
#elif defined(EE_AVR8_TIMER3_COMPA_ISR) && (EE_AVR8_TIMER3_COMPA_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER3_COMPA_vect, EE_AVR8_TIMER3_COMPA_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER3_COMPA_vect)
#endif /* EE_AVR8_TIMER3_COMPA_ISR */

/* TIMER3_COMPB interrupt vector */
#if defined(EE_AVR8_TIMER3_COMPB_ISR) && (EE_AVR8_TIMER3_COMPB_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER3_COMPB_vect, EE_AVR8_TIMER3_COMPB_ISR)
#elif defined(EE_AVR8_TIMER3_COMPB_ISR) && (EE_AVR8_TIMER3_COMPB_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER3_COMPB_vect, EE_AVR8_TIMER3_COMPB_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER3_COMPB_vect)
#endif /* EE_AVR8_TIMER3_COMPB_ISR */

/* TIMER3_COMPC interrupt vector */
#if defined(EE_AVR8_TIMER3_COMPC_ISR) && (EE_AVR8_TIMER3_COMPC_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER3_COMPC_vect, EE_AVR8_TIMER3_COMPC_ISR)
#elif defined(EE_AVR8_TIMER3_COMPC_ISR) && (EE_AVR8_TIMER3_COMPC_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER3_COMPC_vect, EE_AVR8_TIMER3_COMPC_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER3_COMPC_vect)
#endif /* EE_AVR8_TIMER3_COMPC_ISR */

/* TIMER3_OVF interrupt vector */
#if defined(EE_AVR8_TIMER3_OVF_ISR) && (EE_AVR8_TIMER3_OVF_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER3_OVF_vect, EE_AVR8_TIMER3_OVF_ISR)
#elif defined(EE_AVR8_TIMER3_OVF_ISR) && (EE_AVR8_TIMER3_OVF_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER3_OVF_vect, EE_AVR8_TIMER3_OVF_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER3_OVF_vect)
#endif /* EE_AVR8_TIMER3_OVF_ISR */

/* USART1_RX interrupt vector */
#if defined(EE_AVR8_USART1_RX_ISR) && (EE_AVR8_USART1_RX_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(USART1_RX_vect, EE_AVR8_USART1_RX_ISR)
#elif defined(EE_AVR8_USART1_RX_ISR) && (EE_AVR8_USART1_RX_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(USART1_RX_vect, EE_AVR8_USART1_RX_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(USART1_RX_vect)
#endif /* EE_AVR8_USART1_RX_ISR */

/* USART1_UDRE interrupt vector */
#if defined(EE_AVR8_USART1_UDRE_ISR) && (EE_AVR8_USART1_UDRE_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(USART1_UDRE_vect, EE_AVR8_USART1_UDRE_ISR)
#elif defined(EE_AVR8_USART1_UDRE_ISR) && (EE_AVR8_USART1_UDRE_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(USART1_UDRE_vect, EE_AVR8_USART1_UDRE_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(USART1_UDRE_vect)
#endif /* EE_AVR8_USART1_UDRE_ISR */

/* USART1_TX interrupt vector */
#if defined(EE_AVR8_USART1_TX_ISR) && (EE_AVR8_USART1_TX_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(USART1_TX_vect, EE_AVR8_USART1_TX_ISR)
#elif defined(EE_AVR8_USART1_TX_ISR) && (EE_AVR8_USART1_TX_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(USART1_TX_vect, EE_AVR8_USART1_TX_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(USART1_TX_vect)
#endif /* EE_AVR8_USART1_TX_ISR */
#endif	/* __AVR_ATmega128__ || __AVR_ATmega1281__ */

/* TWI interrupt vector */
#if defined(EE_AVR8_TWI_ISR) && (EE_AVR8_TWI_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TWI_vect, EE_AVR8_TWI_ISR)
#elif defined(EE_AVR8_TWI_ISR) && (EE_AVR8_TWI_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TWI_vect, EE_AVR8_TWI_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TWI_vect)
#endif /* EE_AVR8_TWI_ISR */

/* SPM_READY interrupt vector */
#if defined(EE_AVR8_SPM_READY_ISR) && (EE_AVR8_SPM_READY_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(SPM_READY_vect, EE_AVR8_SPM_READY_ISR)
#elif defined(EE_AVR8_SPM_READY_ISR) && (EE_AVR8_SPM_READY_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(SPM_READY_vect, EE_AVR8_SPM_READY_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(SPM_READY_vect)
#endif /* EE_AVR8_SPM_READY_ISR */

#if	( defined(__AVR_ATmega328__) || defined(__AVR_ATmega328P__) )
/* TIMER4_CAPT interrupt vector */
#if defined(EE_AVR8_TIMER4_CAPT_ISR) && (EE_AVR8_TIMER4_CAPT_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER4_CAPT_vect, EE_AVR8_TIMER4_CAPT_ISR)
#elif defined(EE_AVR8_TIMER4_CAPT_ISR) && (EE_AVR8_TIMER4_CAPT_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER4_CAPT_vect, EE_AVR8_TIMER4_CAPT_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER4_CAPT_vect)
#endif /* EE_AVR8_TIMER4_CAPT_ISR */

/* TIMER4_COMPA interrupt vector */
#if defined(EE_AVR8_TIMER4_COMPA_ISR) && (EE_AVR8_TIMER4_COMPA_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER4_COMPA_vect, EE_AVR8_TIMER4_COMPA_ISR)
#elif defined(EE_AVR8_TIMER4_COMPA_ISR) && (EE_AVR8_TIMER4_COMPA_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER4_COMPA_vect, EE_AVR8_TIMER4_COMPA_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER4_COMPA_vect)
#endif /* EE_AVR8_TIMER4_COMPA_ISR */

/* TIMER4_COMPB interrupt vector */
#if defined(EE_AVR8_TIMER4_COMPB_ISR) && (EE_AVR8_TIMER4_COMPB_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER4_COMPB_vect, EE_AVR8_TIMER4_COMPB_ISR)
#elif defined(EE_AVR8_TIMER4_COMPB_ISR) && (EE_AVR8_TIMER4_COMPB_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER4_COMPB_vect, EE_AVR8_TIMER4_COMPB_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER4_COMPB_vect)
#endif /* EE_AVR8_TIMER4_COMPB_ISR */

/* TIMER4_COMPC interrupt vector */
#if defined(EE_AVR8_TIMER4_COMPC_ISR) && (EE_AVR8_TIMER4_COMPC_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER4_COMPC_vect, EE_AVR8_TIMER4_COMPC_ISR)
#elif defined(EE_AVR8_TIMER4_COMPC_ISR) && (EE_AVR8_TIMER4_COMPC_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER4_COMPC_vect, EE_AVR8_TIMER4_COMPC_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER4_COMPC_vect)
#endif /* EE_AVR8_TIMER4_COMPC_ISR */

/* TIMER4_OVF interrupt vector */
#if defined(EE_AVR8_TIMER4_OVF_ISR) && (EE_AVR8_TIMER4_OVF_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER4_OVF_vect, EE_AVR8_TIMER4_OVF_ISR)
#elif defined(EE_AVR8_TIMER4_OVF_ISR) && (EE_AVR8_TIMER4_OVF_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER4_OVF_vect, EE_AVR8_TIMER4_OVF_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER4_OVF_vect)
#endif /* EE_AVR8_TIMER4_OVF_ISR */

/* TIMER5_CAPT interrupt vector */
#if defined(EE_AVR8_TIMER5_CAPT_ISR) && (EE_AVR8_TIMER5_CAPT_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER5_CAPT_vect, EE_AVR8_TIMER5_CAPT_ISR)
#elif defined(EE_AVR8_TIMER5_CAPT_ISR) && (EE_AVR8_TIMER5_CAPT_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER5_CAPT_vect, EE_AVR8_TIMER5_CAPT_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER5_CAPT_vect)
#endif /* EE_AVR8_TIMER5_CAPT_ISR */

/* TIMER5_COMPA interrupt vector */
#if defined(EE_AVR8_TIMER5_COMPA_ISR) && (EE_AVR8_TIMER5_COMPA_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER5_COMPA_vect, EE_AVR8_TIMER5_COMPA_ISR)
#elif defined(EE_AVR8_TIMER5_COMPA_ISR) && (EE_AVR8_TIMER5_COMPA_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER5_COMPA_vect, EE_AVR8_TIMER5_COMPA_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER5_COMPA_vect)
#endif /* EE_AVR8_TIMER5_COMPA_ISR */

/* TIMER5_COMPB interrupt vector */
#if defined(EE_AVR8_TIMER5_COMPB_ISR) && (EE_AVR8_TIMER5_COMPB_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER5_COMPB_vect, EE_AVR8_TIMER5_COMPB_ISR)
#elif defined(EE_AVR8_TIMER5_COMPB_ISR) && (EE_AVR8_TIMER5_COMPB_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER5_COMPB_vect, EE_AVR8_TIMER5_COMPB_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER5_COMPB_vect)
#endif /* EE_AVR8_TIMER5_COMPB_ISR */

/* TIMER5_COMPC interrupt vector */
#if defined(EE_AVR8_TIMER5_COMPC_ISR) && (EE_AVR8_TIMER5_COMPC_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER5_COMPC_vect, EE_AVR8_TIMER5_COMPC_ISR)
#elif defined(EE_AVR8_TIMER5_COMPC_ISR) && (EE_AVR8_TIMER5_COMPC_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER5_COMPC_vect, EE_AVR8_TIMER5_COMPC_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER5_COMPC_vect)
#endif /* EE_AVR8_TIMER5_COMPC_ISR */

/* TIMER5_OVF interrupt vector */
#if defined(EE_AVR8_TIMER5_OVF_ISR) && (EE_AVR8_TIMER5_OVF_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(TIMER5_OVF_vect, EE_AVR8_TIMER5_OVF_ISR)
#elif defined(EE_AVR8_TIMER5_OVF_ISR) && (EE_AVR8_TIMER5_OVF_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(TIMER5_OVF_vect, EE_AVR8_TIMER5_OVF_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(TIMER5_OVF_vect)
#endif /* EE_AVR8_TIMER5_OVF_ISR */

/* USART2_RX interrupt vector */
#if defined(EE_AVR8_USART2_RX_ISR) && (EE_AVR8_USART2_RX_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(USART2_RX_vect, EE_AVR8_USART2_RX_ISR)
#elif defined(EE_AVR8_USART2_RX_ISR) && (EE_AVR8_USART2_RX_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(USART2_RX_vect, EE_AVR8_USART2_RX_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(USART2_RX_vect)
#endif /* EE_AVR8_USART2_RX_ISR */

/* USART2_UDRE interrupt vector */
#if defined(EE_AVR8_USART2_UDRE_ISR) && (EE_AVR8_USART2_UDRE_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(USART2_UDRE_vect, EE_AVR8_USART2_UDRE_ISR)
#elif defined(EE_AVR8_USART2_UDRE_ISR) && (EE_AVR8_USART2_UDRE_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(USART2_UDRE_vect, EE_AVR8_USART2_UDRE_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(USART2_UDRE_vect)
#endif /* EE_AVR8_USART2_UDRE_ISR */

/* USART2_TX interrupt vector */
#if defined(EE_AVR8_USART2_TX_ISR) && (EE_AVR8_USART2_TX_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(USART2_TX_vect, EE_AVR8_USART2_TX_ISR)
#elif defined(EE_AVR8_USART2_TX_ISR) && (EE_AVR8_USART2_TX_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(USART2_TX_vect, EE_AVR8_USART2_TX_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(USART2_TX_vect)
#endif /* EE_AVR8_USART2_TX_ISR */

/* USART3_RX interrupt vector */
#if defined(EE_AVR8_USART3_RX_ISR) && (EE_AVR8_USART3_RX_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(USART3_RX_vect, EE_AVR8_USART3_RX_ISR)
#elif defined(EE_AVR8_USART3_RX_ISR) && (EE_AVR8_USART3_RX_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(USART3_RX_vect, EE_AVR8_USART3_RX_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(USART3_RX_vect)
#endif /* EE_AVR8_USART3_RX_ISR */

/* USART3_UDRE interrupt vector */
#if defined(EE_AVR8_USART3_UDRE_ISR) && (EE_AVR8_USART3_UDRE_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(USART3_UDRE_vect, EE_AVR8_USART3_UDRE_ISR)
#elif defined(EE_AVR8_USART3_UDRE_ISR) && (EE_AVR8_USART3_UDRE_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(USART3_UDRE_vect, EE_AVR8_USART3_UDRE_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(USART3_UDRE_vect)
#endif /* EE_AVR8_USART3_UDRE_ISR */

/* USART3_TX interrupt vector */
#if defined(EE_AVR8_USART3_TX_ISR) && (EE_AVR8_USART3_TX_ISR_CAT == 2)
EE_AVR8_ISR2_DEFINITION(USART3_TX_vect, EE_AVR8_USART3_TX_ISR)
#elif defined(EE_AVR8_USART3_TX_ISR) && (EE_AVR8_USART3_TX_ISR_CAT == 1)
EE_AVR8_ISR1_DEFINITION(USART3_TX_vect, EE_AVR8_USART3_TX_ISR)
#elif defined(__MCU_EXTENDED_STATUS__)
EE_AVR8_ISR_NOT_DEFINED(USART3_TX_vect)
#endif /* EE_AVR8_USART3_TX_ISR */
#endif	/* __AVR_ATmega328__ || __AVR_ATmega328P__ */